@@ -19,13 +19,14 @@
 package io.undertow;
 
 import io.undertow.protocols.ssl.UndertowXnioSsl;
+import io.undertow.server.ConnectorStatistics;
 import io.undertow.server.DefaultByteBufferPool;
 import io.undertow.server.HttpHandler;
+import io.undertow.server.OpenListener;
 import io.undertow.server.protocol.ajp.AjpOpenListener;
 import io.undertow.server.protocol.http.AlpnOpenListener;
 import io.undertow.server.protocol.http.HttpOpenListener;
 import io.undertow.server.protocol.http2.Http2OpenListener;
-import io.undertow.server.protocol.spdy.SpdyOpenListener;
 import org.xnio.ChannelListener;
 import org.xnio.ChannelListeners;
 import org.xnio.IoUtils;
@@ -37,6 +38,7 @@ import org.xnio.StreamConnection;
 import org.xnio.Xnio;
 import org.xnio.XnioWorker;
 import org.xnio.channels.AcceptingChannel;
+import org.xnio.ssl.JsseSslUtils;
 import org.xnio.ssl.SslConnection;
 import org.xnio.ssl.XnioSsl;
 
@@ -45,7 +47,10 @@ import javax.net.ssl.SSLContext;
 import javax.net.ssl.TrustManager;
 import java.net.Inet4Address;
 import java.net.InetSocketAddress;
+import java.net.SocketAddress;
+import java.security.SecureRandom;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.List;
 
 /**
@@ -57,28 +62,40 @@ import java.util.List;
 public final class Undertow {
 
     private final int bufferSize;
-    private final int buffersPerRegion;
     private final int ioThreads;
     private final int workerThreads;
     private final boolean directBuffers;
     private final List<ListenerConfig> listeners = new ArrayList<>();
+    private volatile List<ListenerInfo> listenerInfo;
     private final HttpHandler rootHandler;
     private final OptionMap workerOptions;
     private final OptionMap socketOptions;
     private final OptionMap serverOptions;
 
+    /**
+     * Will be true when a {@link XnioWorker} instance was NOT provided to the {@link Builder}.
+     * When true, a new worker will be created during {@link Undertow#start()},
+     * and shutdown when {@link Undertow#stop()} is called.
+     *
+     * Will be false when a {@link XnioWorker} instance was provided to the {@link Builder}.
+     * When false, the provided {@link #worker} will be used instead of creating a new one in {@link Undertow#start()}.
+     * Also, when false, the {@link #worker} will NOT be shutdown when {@link Undertow#stop()} is called.
+     */
+    private final boolean internalWorker;
+
     private XnioWorker worker;
     private List<AcceptingChannel<? extends StreamConnection>> channels;
     private Xnio xnio;
 
     private Undertow(Builder builder) {
         this.bufferSize = builder.bufferSize;
-        this.buffersPerRegion = builder.buffersPerRegion;
         this.ioThreads = builder.ioThreads;
         this.workerThreads = builder.workerThreads;
         this.directBuffers = builder.directBuffers;
         this.listeners.addAll(builder.listeners);
         this.rootHandler = builder.handler;
+        this.worker = builder.worker;
+        this.internalWorker = builder.worker == null;
         this.workerOptions = builder.workerOptions.getMap();
         this.socketOptions = builder.socketOptions.getMap();
         this.serverOptions = builder.serverOptions.getMap();
@@ -92,22 +109,25 @@ public final class Undertow {
     }
 
     public synchronized void start() {
+        UndertowLogger.ROOT_LOGGER.debugf("starting undertow server %s", this);
         xnio = Xnio.getInstance(Undertow.class.getClassLoader());
         channels = new ArrayList<>();
         try {
-            worker = xnio.createWorker(OptionMap.builder()
-                    .set(Options.WORKER_IO_THREADS, ioThreads)
-                    .set(Options.CONNECTION_HIGH_WATER, 1000000)
-                    .set(Options.CONNECTION_LOW_WATER, 1000000)
-                    .set(Options.WORKER_TASK_CORE_THREADS, workerThreads)
-                    .set(Options.WORKER_TASK_MAX_THREADS, workerThreads)
-                    .set(Options.TCP_NODELAY, true)
-                    .set(Options.CORK, true)
-                    .addAll(workerOptions)
-                    .getMap());
+            if (internalWorker) {
+                worker = xnio.createWorker(OptionMap.builder()
+                        .set(Options.WORKER_IO_THREADS, ioThreads)
+                        .set(Options.CONNECTION_HIGH_WATER, 1000000)
+                        .set(Options.CONNECTION_LOW_WATER, 1000000)
+                        .set(Options.WORKER_TASK_CORE_THREADS, workerThreads)
+                        .set(Options.WORKER_TASK_MAX_THREADS, workerThreads)
+                        .set(Options.TCP_NODELAY, true)
+                        .set(Options.CORK, true)
+                        .addAll(workerOptions)
+                        .getMap());
+            }
 
             OptionMap socketOptions = OptionMap.builder()
-                    .set(Options.WORKER_IO_THREADS, ioThreads)
+                    .set(Options.WORKER_IO_THREADS, worker.getIoThreadCount())
                     .set(Options.TCP_NODELAY, true)
                     .set(Options.REUSE_ADDRESSES, true)
                     .set(Options.BALANCING_TOKENS, 1)
@@ -116,10 +136,17 @@ public final class Undertow {
                     .addAll(this.socketOptions)
                     .getMap();
 
+            OptionMap serverOptions = OptionMap.builder()
+                    .set(UndertowOptions.NO_REQUEST_TIMEOUT, 60 * 1000)
+                    .addAll(this.serverOptions)
+                    .getMap();
+
 
             ByteBufferPool buffers = new DefaultByteBufferPool(directBuffers, bufferSize, -1, 4);
 
+            listenerInfo = new ArrayList<>();
             for (ListenerConfig listener : listeners) {
+                UndertowLogger.ROOT_LOGGER.debugf("Configuring listener with protocol %s for interface %s and port %s", listener.type, listener.host, listener.port);
                 final HttpHandler rootHandler = listener.rootHandler != null ? listener.rootHandler : this.rootHandler;
                 if (listener.type == ListenerType.AJP) {
                     AjpOpenListener openListener = new AjpOpenListener(buffers, serverOptions);
@@ -128,6 +155,7 @@ public final class Undertow {
                     AcceptingChannel<? extends StreamConnection> server = worker.createStreamConnectionServer(new InetSocketAddress(Inet4Address.getByName(listener.host), listener.port), acceptListener, socketOptions);
                     server.resumeAccepts();
                     channels.add(server);
+                    listenerInfo.add(new ListenerInfo("ajp", server.getLocalAddress(), null, openListener));
                 } else {
                     OptionMap undertowOptions = OptionMap.builder().set(UndertowOptions.BUFFER_PIPELINED_DATA, true).addAll(serverOptions).getMap();
                     if (listener.type == ListenerType.HTTP) {
@@ -137,21 +165,16 @@ public final class Undertow {
                         AcceptingChannel<? extends StreamConnection> server = worker.createStreamConnectionServer(new InetSocketAddress(Inet4Address.getByName(listener.host), listener.port), acceptListener, socketOptions);
                         server.resumeAccepts();
                         channels.add(server);
+                        listenerInfo.add(new ListenerInfo("http", server.getLocalAddress(), null, openListener));
                     } else if (listener.type == ListenerType.HTTPS) {
-                        ChannelListener<StreamConnection> openListener;
+                        OpenListener openListener;
 
                         HttpOpenListener httpOpenListener = new HttpOpenListener(buffers, undertowOptions);
                         httpOpenListener.setRootHandler(rootHandler);
 
-                        boolean spdy = serverOptions.get(UndertowOptions.ENABLE_SPDY, false);
                         boolean http2 = serverOptions.get(UndertowOptions.ENABLE_HTTP2, false);
-                        if(spdy || http2) {
+                        if(http2) {
                             AlpnOpenListener alpn = new AlpnOpenListener(buffers, undertowOptions, httpOpenListener);
-                            if(spdy) {
-                                SpdyOpenListener spdyListener = new SpdyOpenListener(buffers, new DefaultByteBufferPool(false, 1024, -1, 2, 0), undertowOptions);
-                                spdyListener.setRootHandler(rootHandler);
-                                alpn.addProtocol(SpdyOpenListener.SPDY_3_1, spdyListener, 5);
-                            }
                             if(http2) {
                                 Http2OpenListener http2Listener = new Http2OpenListener(buffers, undertowOptions);
                                 http2Listener.setRootHandler(rootHandler);
@@ -167,11 +190,13 @@ public final class Undertow {
                         if (listener.sslContext != null) {
                             xnioSsl = new UndertowXnioSsl(xnio, OptionMap.create(Options.USE_DIRECT_BUFFERS, true), listener.sslContext);
                         } else {
-                            xnioSsl = xnio.getSslProvider(listener.keyManagers, listener.trustManagers, OptionMap.create(Options.USE_DIRECT_BUFFERS, true));
+
+                            xnioSsl = new UndertowXnioSsl(xnio, OptionMap.create(Options.USE_DIRECT_BUFFERS, true), JsseSslUtils.createSSLContext(listener.keyManagers, listener.trustManagers, new SecureRandom(), OptionMap.create(Options.USE_DIRECT_BUFFERS, true)));
                         }
                         AcceptingChannel<SslConnection> sslServer = xnioSsl.createSslConnectionServer(worker, new InetSocketAddress(Inet4Address.getByName(listener.host), listener.port), (ChannelListener) acceptListener, socketOptions);
                         sslServer.resumeAccepts();
                         channels.add(sslServer);
+                        listenerInfo.add(new ListenerInfo("https", sslServer.getLocalAddress(), listener.sslContext, openListener));
                     }
                 }
 
@@ -183,17 +208,43 @@ public final class Undertow {
     }
 
     public synchronized void stop() {
-        for (AcceptingChannel<? extends StreamConnection> channel : channels) {
-            IoUtils.safeClose(channel);
+        UndertowLogger.ROOT_LOGGER.debugf("stopping undertow server %s", this);
+        if (channels != null) {
+            for (AcceptingChannel<? extends StreamConnection> channel : channels) {
+                IoUtils.safeClose(channel);
+            }
+            channels = null;
+        }
+
+        /*
+         * Only shutdown the worker if it was created during start()
+         */
+        if (internalWorker && worker != null) {
+            worker.shutdownNow();
+            worker = null;
         }
-        channels = null;
-        worker.shutdownNow();
-        worker = null;
         xnio = null;
+        listenerInfo = null;
+    }
+
+    public Xnio getXnio() {
+        return xnio;
+    }
+
+    public XnioWorker getWorker() {
+        return worker;
+    }
+
+    public List<ListenerInfo> getListenerInfo() {
+        if(listenerInfo == null) {
+            throw UndertowMessages.MESSAGES.serverNotStarted();
+        }
+        return Collections.unmodifiableList(listenerInfo);
     }
 
 
-    public static enum ListenerType {
+
+    public enum ListenerType {
         HTTP,
         HTTPS,
         AJP
@@ -232,12 +283,12 @@ public final class Undertow {
     public static final class Builder {
 
         private int bufferSize;
-        private int buffersPerRegion;
         private int ioThreads;
         private int workerThreads;
         private boolean directBuffers;
         private final List<ListenerConfig> listeners = new ArrayList<>();
         private HttpHandler handler;
+        private XnioWorker worker;
 
         private final OptionMap.Builder workerOptions = OptionMap.builder();
         private final OptionMap.Builder socketOptions = OptionMap.builder();
@@ -252,18 +303,15 @@ public final class Undertow {
                 //use 512b buffers
                 directBuffers = false;
                 bufferSize = 512;
-                buffersPerRegion = 10;
             } else if (maxMemory < 128 * 1024 * 1024) {
                 //use 1k buffers
                 directBuffers = true;
                 bufferSize = 1024;
-                buffersPerRegion = 10;
             } else {
                 //use 16k buffers for best performance
                 //as 16k is generally the max amount of data that can be sent in a single write() call
                 directBuffers = true;
                 bufferSize = 1024 * 16;
-                buffersPerRegion = 20;
             }
 
         }
@@ -328,8 +376,8 @@ public final class Undertow {
             return this;
         }
 
+        @Deprecated
         public Builder setBuffersPerRegion(final int buffersPerRegion) {
-            this.buffersPerRegion = buffersPerRegion;
             return this;
         }
 
@@ -367,6 +415,63 @@ public final class Undertow {
             workerOptions.set(option, value);
             return this;
         }
+
+        /**
+         * When null (the default), a new {@link XnioWorker} will be created according
+         * to the various worker-related configuration (ioThreads, workerThreads, workerOptions)
+         * when {@link Undertow#start()} is called.
+         * Additionally, this newly created worker will be shutdown when {@link Undertow#stop()} is called.
+         * <br/>
+         *
+         * When non-null, the provided {@link XnioWorker} will be reused instead of creating a new {@link XnioWorker}
+         * when {@link Undertow#start()} is called.
+         * Additionally, the provided {@link XnioWorker} will NOT be shutdown when {@link Undertow#stop()} is called.
+         * Essentially, the lifecycle of the provided worker must be maintained outside of the {@link Undertow} instance.
+         */
+        public <T> Builder setWorker(XnioWorker worker) {
+            this.worker = worker;
+            return this;
+        }
+    }
+
+    public static class ListenerInfo {
+
+        private final String protcol;
+        private final SocketAddress address;
+        private final SSLContext sslContext;
+        private final OpenListener openListener;
+
+        public ListenerInfo(String protcol, SocketAddress address, SSLContext sslContext, OpenListener openListener) {
+            this.protcol = protcol;
+            this.address = address;
+            this.sslContext = sslContext;
+            this.openListener = openListener;
+        }
+
+        public String getProtcol() {
+            return protcol;
+        }
+
+        public SocketAddress getAddress() {
+            return address;
+        }
+
+        public SSLContext getSslContext() {
+            return sslContext;
+        }
+
+        public ConnectorStatistics getConnectorStatistics() {
+            return openListener.getConnectorStatistics();
+        }
+
+        @Override
+        public String toString() {
+            return "ListenerInfo{" +
+                    "protcol='" + protcol + '\'' +
+                    ", address=" + address +
+                    ", sslContext=" + sslContext +
+                    '}';
+        }
     }
 
 }
