@@ -21,16 +21,20 @@ package io.undertow;
 import java.io.IOException;
 import java.net.SocketAddress;
 import java.nio.channels.ClosedChannelException;
+import javax.net.ssl.SSLHandshakeException;
+import javax.net.ssl.SSLPeerUnverifiedException;
 
-import io.undertow.predicate.PredicateBuilder;
-import io.undertow.protocols.http2.HpackException;
-import io.undertow.server.handlers.builder.HandlerBuilder;
 import org.jboss.logging.Messages;
 import org.jboss.logging.annotations.Cause;
 import org.jboss.logging.annotations.Message;
 import org.jboss.logging.annotations.MessageBundle;
-
-import javax.net.ssl.SSLPeerUnverifiedException;
+import io.undertow.predicate.PredicateBuilder;
+import io.undertow.protocols.http2.HpackException;
+import io.undertow.security.api.AuthenticationMechanism;
+import io.undertow.server.handlers.builder.HandlerBuilder;
+import io.undertow.server.protocol.http.HttpRequestParser;
+import io.undertow.util.HttpString;
+import io.undertow.util.ParameterLimitException;
 
 /**
  * @author Stuart Douglas
@@ -64,8 +68,8 @@ public interface UndertowMessages {
     @Message(id = 9, value = "Path must be specified")
     IllegalArgumentException pathMustBeSpecified();
 
-    @Message(id = 10, value = "Session not found %s")
-    IllegalStateException sessionNotFound(final String session);
+    @Message(id = 10, value = "Session is invalid %s")
+    IllegalStateException sessionIsInvalid(String sessionId);
 
     @Message(id = 11, value = "Session manager must not be null")
     IllegalStateException sessionManagerMustNotBeNull();
@@ -146,10 +150,10 @@ public interface UndertowMessages {
     String authenticationFailed(final String userName);
 
     @Message(id = 39, value = "To many query parameters, cannot have more than %s query parameters")
-    RuntimeException tooManyQueryParameters(int noParams);
+    HttpRequestParser.BadRequestException tooManyQueryParameters(int noParams);
 
     @Message(id = 40, value = "To many headers, cannot have more than %s header")
-    RuntimeException tooManyHeaders(int noParams);
+    String tooManyHeaders(int noParams);
 
     @Message(id = 41, value = "Channel is closed")
     ClosedChannelException channelIsClosed();
@@ -170,7 +174,7 @@ public interface UndertowMessages {
     IllegalStateException tooManyCookies(int maxCookies);
 
     @Message(id = 47, value = "The number of parameters exceeded the maximum of %s")
-    IllegalStateException tooManyParameters(int maxValues);
+    ParameterLimitException tooManyParameters(int maxValues);
 
     @Message(id = 48, value = "No request is currently active")
     IllegalStateException noRequestActive();
@@ -293,11 +297,11 @@ public interface UndertowMessages {
     @Message(id = 88, value = "SPDY control frames cannot have body content")
     IOException controlFrameCannotHaveBodyContent();
 
-    @Message(id = 89, value = "SPDY not supported")
-    IOException spdyNotSupported();
+//    @Message(id = 89, value = "SPDY not supported")
+//    IOException spdyNotSupported();
 
-    @Message(id = 90, value = "Jetty NPN not available")
-    IOException jettyNPNNotAvailable();
+    @Message(id = 90, value = "No ALPN implementation available (tried Jetty ALPN and JDK9)")
+    IOException alpnNotAvailable();
 
     @Message(id = 91, value = "Buffer has already been freed")
     IllegalStateException bufferAlreadyFreed();
@@ -305,8 +309,8 @@ public interface UndertowMessages {
     @Message(id = 92, value = "A SPDY header was too large to fit in a response buffer, if you want to support larger headers please increase the buffer size")
     IllegalStateException headersTooLargeToFitInHeapBuffer();
 
-    @Message(id = 93, value = "A SPDY stream was reset by the remote endpoint")
-    IOException spdyStreamWasReset();
+//    @Message(id = 93, value = "A SPDY stream was reset by the remote endpoint")
+//    IOException spdyStreamWasReset();
 
     @Message(id = 94, value = "Blocking await method called from IO thread. Blocking IO must be dispatched to a worker thread or deadlocks will result.")
     IOException awaitCalledFromIoThread();
@@ -434,4 +438,70 @@ public interface UndertowMessages {
 
     @Message(id = 135, value = "renegotiation failed")
     IllegalStateException rengotiationFailed();
+
+    @Message(id = 136, value = "User agent charset string must have an even number of items, in the form pattern,charset,pattern,charset,... Instead got: %s")
+    IllegalArgumentException userAgentCharsetMustHaveEvenNumberOfItems(String supplied);
+
+    @Message(id = 137, value = "Could not find the datasource called %s")
+    IllegalArgumentException datasourceNotFound(String ds);
+
+    @Message(id = 138, value = "Server not started")
+    IllegalStateException serverNotStarted();
+
+    @Message(id = 139, value = "Exchange already complete")
+    IllegalStateException exchangeAlreadyComplete();
+
+    @Message(id = 140, value = "Initial SSL/TLS data is not a handshake record")
+    SSLHandshakeException notHandshakeRecord();
+
+    @Message(id = 141, value = "Initial SSL/TLS handshake record is invalid")
+    SSLHandshakeException invalidHandshakeRecord();
+
+    @Message(id = 142, value = "Initial SSL/TLS handshake spans multiple records")
+    SSLHandshakeException multiRecordSSLHandshake();
+
+    @Message(id = 143, value = "Expected \"client hello\" record")
+    SSLHandshakeException expectedClientHello();
+
+    @Message(id = 144, value = "Expected server hello")
+    SSLHandshakeException expectedServerHello();
+
+    @Message(id = 145, value = "Too many redirects")
+    IOException tooManyRedirects(@Cause IOException exception);
+
+    @Message(id = 146, value = "HttpServerExchange cannot have both async IO resumed and dispatch() called in the same cycle")
+    IllegalStateException resumedAndDispatched();
+
+    @Message(id = 147, value = "No host header in a HTTP/1.1 request")
+    IOException noHostInHttp11Request();
+
+    @Message(id = 148, value = "Invalid HPack encoding. First byte: %s")
+    HpackException invalidHpackEncoding(byte b);
+
+    @Message(id = 149, value = "HttpString is not allowed to contain newlines. value: %s")
+    IllegalArgumentException newlineNotSupportedInHttpString(String value);
+
+    @Message(id = 150, value = "Pseudo header %s received after receiving normal headers. Pseudo headers must be the first headers in a HTTP/2 header block.")
+    String pseudoHeaderInWrongOrder(HttpString header);
+
+    @Message(id = 151, value = "Expected to receive a continuation frame")
+    String expectedContinuationFrame();
+
+    @Message(id = 152, value = "Incorrect frame size")
+    String incorrectFrameSize();
+
+    @Message(id = 153, value = "Stream id not registered")
+    IllegalStateException streamNotRegistered();
+
+    @Message(id = 154, value = "Mechanism %s returned a null result from sendChallenge()")
+    NullPointerException sendChallengeReturnedNull(AuthenticationMechanism mechanism);
+
+    @Message(id = 155, value = "Framed channel body was set when it was not ready for flush")
+    IllegalStateException bodyIsSetAndNotReadyForFlush();
+
+    @Message(id = 156, value = "Invalid GZIP header")
+    IOException invalidGzipHeader();
+
+    @Message(id = 157, value = "Invalid GZIP footer")
+    IOException invalidGZIPFooter();
 }
