@@ -28,6 +28,8 @@ import java.nio.channels.ClosedChannelException;
 import java.nio.channels.FileChannel;
 import java.util.concurrent.TimeUnit;
 import java.util.zip.Deflater;
+
+import io.undertow.server.Connectors;
 import org.xnio.IoUtils;
 import io.undertow.connector.PooledByteBuffer;
 import org.xnio.XnioIoThread;
@@ -85,6 +87,7 @@ public class DeflatingStreamSinkConduit implements StreamSinkConduit {
         this.currentBuffer = exchange.getConnection().getByteBufferPool().allocate();
         this.exchange = exchange;
         this.conduitFactory = conduitFactory;
+        setWriteReadyHandler(new WriteReadyHandler.ChannelListenerHandler<>(Connectors.getConduitSinkChannel(exchange)));
     }
 
     @Override
@@ -101,7 +104,7 @@ public class DeflatingStreamSinkConduit implements StreamSinkConduit {
             }
             //we may already have some input, if so compress it
             if (!deflater.needsInput()) {
-                deflateData();
+                deflateData(false);
                 if (!deflater.needsInput()) {
                     return 0;
                 }
@@ -110,7 +113,8 @@ public class DeflatingStreamSinkConduit implements StreamSinkConduit {
             src.get(data);
             preDeflate(data);
             deflater.setInput(data);
-            deflateData();
+            Connectors.updateResponseBytesSent(exchange, 0 - data.length);
+            deflateData(false);
             return data.length;
         } catch (IOException e) {
             freeBuffer();
@@ -299,7 +303,7 @@ public class DeflatingStreamSinkConduit implements StreamSinkConduit {
                         }
                         //if the deflater has not been fully flushed we need to flush it
                         if (!deflater.finished()) {
-                            deflateData();
+                            deflateData(false);
                             //if could not fully flush
                             if (!deflater.finished()) {
                                 return false;
@@ -310,6 +314,7 @@ public class DeflatingStreamSinkConduit implements StreamSinkConduit {
                             state |= WRITTEN_TRAILER;
                             byte[] data = getTrailer();
                             if (data != null) {
+                                Connectors.updateResponseBytesSent(exchange, data.length);
                                 if(additionalBuffer != null) {
                                     byte[] newData = new byte[additionalBuffer.remaining() + data.length];
                                     int pos = 0;
@@ -351,7 +356,22 @@ public class DeflatingStreamSinkConduit implements StreamSinkConduit {
                         }
                     }
                 } else {
-                    return performFlushIfRequired();
+                    if(allAreClear(state, FLUSHING_BUFFER)) {
+                        if (next == null) {
+                            nextCreated = true;
+                            this.next = createNextChannel();
+                        }
+                        deflateData(true);
+                        if(allAreClear(state, FLUSHING_BUFFER)) {
+                            //deflateData can cause this to be change
+                            currentBuffer.getBuffer().flip();
+                            this.state |= FLUSHING_BUFFER;
+                        }
+                    }
+                    if(!performFlushIfRequired()) {
+                        return false;
+                    }
+                    return next.flush();
                 }
             } finally {
                 if (nextCreated) {
@@ -432,29 +452,31 @@ public class DeflatingStreamSinkConduit implements StreamSinkConduit {
      *
      * @throws IOException
      */
-    private void deflateData() throws IOException {
+    private void deflateData(boolean force) throws IOException {
         //we don't need to flush here, as this should have been called already by the time we get to
         //this point
         boolean nextCreated = false;
-        try {
+        try (PooledByteBuffer arrayPooled = this.exchange.getConnection().getByteBufferPool().getArrayBackedPool().allocate()) {
             PooledByteBuffer pooled = this.currentBuffer;
             final ByteBuffer outputBuffer = pooled.getBuffer();
 
             final boolean shutdown = anyAreSet(state, SHUTDOWN);
-
-            byte[] buffer = new byte[1024]; //TODO: we should pool this and make it configurable or something
-            while (!deflater.needsInput() || (shutdown && !deflater.finished())) {
-                int count = deflater.deflate(buffer);
+            ByteBuffer buf = arrayPooled.getBuffer();
+            while (force || !deflater.needsInput() || (shutdown && !deflater.finished())) {
+                int count = deflater.deflate(buf.array(), buf.arrayOffset(), buf.remaining(), force ? Deflater.SYNC_FLUSH: Deflater.NO_FLUSH);
+                Connectors.updateResponseBytesSent(exchange, count);
                 if (count != 0) {
                     int remaining = outputBuffer.remaining();
                     if (remaining > count) {
-                        outputBuffer.put(buffer, 0, count);
+                        outputBuffer.put(buf.array(), buf.arrayOffset(), count);
                     } else {
                         if (remaining == count) {
-                            outputBuffer.put(buffer, 0, count);
+                            outputBuffer.put(buf.array(), buf.arrayOffset(), count);
                         } else {
-                            outputBuffer.put(buffer, 0, remaining);
-                            additionalBuffer = ByteBuffer.wrap(buffer, remaining, count - remaining);
+                            outputBuffer.put(buf.array(), buf.arrayOffset(), remaining);
+                            additionalBuffer = ByteBuffer.allocate(count - remaining);
+                            additionalBuffer.put(buf.array(), buf.arrayOffset() + remaining, count - remaining);
+                            additionalBuffer.flip();
                         }
                         outputBuffer.flip();
                         this.state |= FLUSHING_BUFFER;
@@ -466,6 +488,8 @@ public class DeflatingStreamSinkConduit implements StreamSinkConduit {
                             return;
                         }
                     }
+                } else {
+                    force = false;
                 }
             }
         } finally {
