@@ -23,6 +23,7 @@ import io.undertow.UndertowMessages;
 import io.undertow.UndertowOptions;
 import io.undertow.connector.ByteBufferPool;
 import io.undertow.connector.PooledByteBuffer;
+import io.undertow.server.protocol.ParseTimeoutUpdater;
 import io.undertow.server.protocol.framed.AbstractFramedChannel;
 import io.undertow.server.protocol.framed.FrameHeaderData;
 import io.undertow.server.protocol.http2.Http2OpenListener;
@@ -53,6 +54,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Random;
 import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.TimeUnit;
 import javax.net.ssl.SSLSession;
 
 /**
@@ -137,6 +139,7 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
     private int sendMaxFrameSize = DEFAULT_MAX_FRAME_SIZE;
     private int receiveMaxFrameSize = DEFAULT_MAX_FRAME_SIZE;
     private int unackedReceiveMaxFrameSize = DEFAULT_MAX_FRAME_SIZE; //the old max frame size, this gets updated when our setting frame is acked
+    private final int maxHeaders;
 
     /**
      * How much data we have told the remote endpoint we are prepared to accept.
@@ -173,6 +176,8 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
 
     private final Map<AttachmentKey<?>, Object> attachments = Collections.synchronizedMap(new HashMap<AttachmentKey<?>, Object>());
 
+    private ParseTimeoutUpdater parseTimeoutUpdater;
+
 
     public Http2Channel(StreamConnection connectedStreamChannel, String protocol, ByteBufferPool bufferPool, PooledByteBuffer data, boolean clientSide, boolean fromUpgrade, OptionMap settings) {
         this(connectedStreamChannel, protocol, bufferPool, data, clientSide, fromUpgrade, true, null, settings);
@@ -182,13 +187,14 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
     }
 
     public Http2Channel(StreamConnection connectedStreamChannel, String protocol, ByteBufferPool bufferPool, PooledByteBuffer data, boolean clientSide, boolean fromUpgrade, boolean prefaceRequired, ByteBuffer initialOtherSideSettings, OptionMap settings) {
-        super(connectedStreamChannel, bufferPool, Http2FramePriority.INSTANCE, data, settings);
+        super(connectedStreamChannel, bufferPool, new Http2FramePriority(clientSide ? (fromUpgrade ? 3 : 1) : 2), data, settings);
         streamIdCounter = clientSide ? (fromUpgrade ? 3 : 1) : 2;
 
         pushEnabled = settings.get(UndertowOptions.HTTP2_SETTINGS_ENABLE_PUSH, true);
         this.initialReceiveWindowSize = settings.get(UndertowOptions.HTTP2_SETTINGS_INITIAL_WINDOW_SIZE, DEFAULT_INITIAL_WINDOW_SIZE);
 
         this.protocol = protocol == null ? Http2OpenListener.HTTP2 : protocol;
+        this.maxHeaders = settings.get(UndertowOptions.MAX_HEADERS, clientSide ? -1 : UndertowOptions.DEFAULT_MAX_HEADERS);
 
         encoderHeaderTableSize = settings.get(UndertowOptions.HTTP2_SETTINGS_HEADER_TABLE_SIZE, Hpack.DEFAULT_TABLE_SIZE);
         receiveMaxFrameSize = settings.get(UndertowOptions.HTTP2_SETTINGS_MAX_FRAME_SIZE, DEFAULT_MAX_FRAME_SIZE);
@@ -232,6 +238,32 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
                 throw new RuntimeException(e);
             }
         }
+        int requestParseTimeout = settings.get(UndertowOptions.REQUEST_PARSE_TIMEOUT, -1);
+        int requestIdleTimeout = settings.get(UndertowOptions.NO_REQUEST_TIMEOUT, -1);
+        if(requestIdleTimeout < 0 && requestParseTimeout < 0) {
+            this.parseTimeoutUpdater = null;
+        } else {
+            this.parseTimeoutUpdater = new ParseTimeoutUpdater(this, requestParseTimeout, requestIdleTimeout, new Runnable() {
+                @Override
+                public void run() {
+                    sendGoAway(ERROR_NO_ERROR);
+                    //just to make sure the connection is actually closed we give it 2 seconds
+                    //then we forcibly kill the connection
+                    getIoThread().executeAfter(new Runnable() {
+                        @Override
+                        public void run() {
+                            IoUtils.safeClose(Http2Channel.this);
+                        }
+                    }, 2, TimeUnit.SECONDS);
+                }
+            });
+            this.addCloseTask(new ChannelListener<Http2Channel>() {
+                @Override
+                public void handleEvent(Http2Channel channel) {
+                    parseTimeoutUpdater.close();
+                }
+            });
+        }
     }
 
     private void sendSettings() {
@@ -271,15 +303,26 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
         }
     }
 
-
-
     private void sendPreface() {
         Http2PrefaceStreamSinkChannel preface = new Http2PrefaceStreamSinkChannel(this);
         flushChannelIgnoreFailure(preface);
     }
-
     @Override
     protected AbstractHttp2StreamSourceChannel createChannel(FrameHeaderData frameHeaderData, PooledByteBuffer frameData) throws IOException {
+        AbstractHttp2StreamSourceChannel channel = createChannelImpl(frameHeaderData, frameData);
+        if(channel instanceof Http2StreamSourceChannel) {
+            if (parseTimeoutUpdater != null) {
+                if (channel != null) {
+                    parseTimeoutUpdater.requestStarted();
+                } else if (currentStreams.isEmpty()) {
+                    parseTimeoutUpdater.failedParse();
+                }
+            }
+        }
+        return channel;
+    }
+
+    protected AbstractHttp2StreamSourceChannel createChannelImpl(FrameHeaderData frameHeaderData, PooledByteBuffer frameData) throws IOException {
 
         Http2FrameHeaderParser frameParser = (Http2FrameHeaderParser) frameHeaderData;
         AbstractHttp2StreamSourceChannel channel;
@@ -427,6 +470,7 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
                     }
                 }
                 frameData.close();
+                sendGoAway(ERROR_NO_ERROR);
                 break;
             }
             case FRAME_TYPE_WINDOW_UPDATE: {
@@ -590,8 +634,6 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
                     return false;
                 }
                 initialSendWindowSize = (int) setting.getValue();
-                int difference = initialSendWindowSize - old;
-                sendWindowSize += difference;
             } else if (setting.getId() == Http2Setting.SETTINGS_MAX_FRAME_SIZE) {
                 if(setting.getValue() > MAX_FRAME_SIZE || setting.getValue() < DEFAULT_MAX_FRAME_SIZE) {
                     UndertowLogger.REQUEST_IO_LOGGER.debug("Invalid value received for SETTINGS_MAX_FRAME_SIZE " + setting.getValue());
@@ -701,6 +743,9 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
             return;
         }
         thisGoneAway = true;
+        if(UndertowLogger.REQUEST_IO_LOGGER.isDebugEnabled()) {
+            UndertowLogger.REQUEST_IO_LOGGER.debugf(new ClosedChannelException(), "Sending goaway on channel %s", this);
+        }
         Http2GoAwayStreamSinkChannel goAway = new Http2GoAwayStreamSinkChannel(this, status, lastGoodStreamId);
         try {
             goAway.shutdownWrites();
@@ -823,6 +868,8 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
         }
         if(isLastFrameReceived() && currentStreams.isEmpty()) {
             sendGoAway(ERROR_NO_ERROR);
+        } else if(parseTimeoutUpdater != null && currentStreams.isEmpty()) {
+            parseTimeoutUpdater.connectionIdle();
         }
     }
 
@@ -839,6 +886,10 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
         return decoder;
     }
 
+    int getMaxHeaders() {
+        return maxHeaders;
+    }
+
     int getPaddingBytes() {
         if(paddingRandom == null) {
             return 0;
@@ -900,6 +951,9 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
 
     public void sendRstStream(int streamId, int statusCode) {
         handleRstStream(streamId);
+        if(UndertowLogger.REQUEST_IO_LOGGER.isDebugEnabled()) {
+            UndertowLogger.REQUEST_IO_LOGGER.debugf(new ClosedChannelException(), "Sending rststream on channel %s stream %s", this, streamId);
+        }
         Http2RstStreamSinkChannel channel = new Http2RstStreamSinkChannel(this, streamId, statusCode);
         flushChannelIgnoreFailure(channel);
     }
@@ -996,7 +1050,7 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
         }
     }
 
-    static final class StreamHolder {
+    private static final class StreamHolder {
         boolean sourceClosed = false;
         boolean sinkClosed = false;
         Http2StreamSourceChannel sourceChannel;
