@@ -74,6 +74,7 @@ import static io.undertow.server.handlers.proxy.mod_cluster.MCMPConstants.STICKY
 import static io.undertow.server.handlers.proxy.mod_cluster.MCMPConstants.TIMEOUT;
 import static io.undertow.server.handlers.proxy.mod_cluster.MCMPConstants.TTL;
 import static io.undertow.server.handlers.proxy.mod_cluster.MCMPConstants.TYPE;
+import static io.undertow.server.handlers.proxy.mod_cluster.MCMPConstants.WAITWORKER;
 
 /**
  * The mod cluster management protocol http handler.
@@ -135,7 +136,7 @@ class MCMPHandler implements HttpHandler {
     @Override
     public void handleRequest(HttpServerExchange exchange) throws Exception {
         final HttpString method = exchange.getRequestMethod();
-        if(!HANDLED_METHODS.contains(method)) {
+        if(!handlesMethod(method)) {
             next.handleRequest(exchange);
             return;
         }
@@ -166,6 +167,10 @@ class MCMPHandler implements HttpHandler {
         }
     }
 
+    protected boolean handlesMethod(HttpString method) {
+        return HANDLED_METHODS.contains(method);
+    }
+
     /**
      * Handle a management+ request.
      *
@@ -278,6 +283,8 @@ class MCMPHandler implements HttpHandler {
             } else if (ALIAS.equals(name)) {
                 final String[] alias = value.split(",");
                 hosts = Arrays.asList(alias);
+            } else if(WAITWORKER.equals(name)) {
+                node.setWaitWorker(Integer.parseInt(value));
             } else {
                 processError(TYPESYNTAX, SBADFLD + name + SBADFLD1, exchange);
                 return;
@@ -584,7 +591,7 @@ class MCMPHandler implements HttpHandler {
     /**
      * Process <tt>INFO</tt> request
      *
-     * @throws Exception
+     * @throws IOException
      */
     protected void processInfo(HttpServerExchange exchange) throws IOException {
         final String data = processInfoString();
@@ -627,7 +634,7 @@ class MCMPHandler implements HttpHandler {
      * Process <tt>DUMP</tt> request
      *
      * @param exchange
-     * @throws java.io.IOException
+     * @throws IOException
      */
     protected void processDump(HttpServerExchange exchange) throws IOException {
         final String data = processDumpString();
@@ -707,7 +714,7 @@ class MCMPHandler implements HttpHandler {
     /**
      * If the process is OK, then add 200 HTTP status and its "OK" phrase
      *
-     * @throws Exception
+     * @throws IOException
      */
     static void processOK(HttpServerExchange exchange) throws IOException {
         exchange.setStatusCode(StatusCodes.OK);
