@@ -18,6 +18,14 @@
 
 package io.undertow.util;
 
+import java.lang.reflect.Field;
+import java.lang.reflect.Modifier;
+import java.security.AccessController;
+import java.security.PrivilegedAction;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.Map;
+
 /**
  * NOTE: if you add a new header here you must also add it to {@link io.undertow.server.protocol.http.HttpRequestParser}
  *
@@ -236,6 +244,35 @@ public final class Headers {
     public static final HttpString USERNAME = new HttpString("username");
 
 
+    private static final Map<String, HttpString> HTTP_STRING_MAP;
+
+    static {
+        Map<String, HttpString> map = AccessController.doPrivileged(new PrivilegedAction<Map<String, HttpString>>() {
+            @Override
+            public Map<String, HttpString> run() {
+                Map<String, HttpString> map = new HashMap<>();
+                Field[] fields = Headers.class.getDeclaredFields();
+                for(Field field : fields) {
+                    if(Modifier.isStatic(field.getModifiers()) && field.getType() == HttpString.class) {
+                        field.setAccessible(true);
+                        try {
+                            HttpString result = (HttpString) field.get(null);
+                            map.put(result.toString(), result);
+                        } catch (IllegalAccessException e) {
+                            throw new RuntimeException(e);
+                        }
+                    }
+                }
+                return map;
+            }
+        });
+        HTTP_STRING_MAP = Collections.unmodifiableMap(map);
+    }
+
+    public static HttpString fromCache(String string) {
+        return HTTP_STRING_MAP.get(string);
+    }
+
     /**
      * Extracts a token from a header that has a given key. For instance if the header is
      * <p>
