@@ -1,5 +1,7 @@
 package io.undertow.util;
 
+import java.util.Locale;
+
 import org.junit.Assert;
 import org.junit.Test;
 
@@ -7,7 +9,7 @@ public class LocaleUtilsTestCase {
 
     @Test
     public void testGetLocaleFromInvalidString() throws Exception {
-        Assert.assertNull(LocaleUtils.getLocaleFromString("-"));
+        Assert.assertEquals(LocaleUtils.getLocaleFromString("-"), new Locale(""));
     }
 
 }
