@@ -71,6 +71,10 @@ public class ServletPathMatches {
         this.resourceManager = deployment.getDeploymentInfo().getResourceManager();
     }
 
+    public void initData(){
+        getData();
+    }
+
     public ServletChain getServletHandlerByName(final String name) {
         return getData().getServletHandlerByName(name);
     }
@@ -247,6 +251,9 @@ public class ServletPathMatches {
                     //an extension match based servlet
                     String ext = path.substring(2);
                     extensionMatches.add(ext);
+                    if(extensionServlets.containsKey(ext)) {
+                        throw UndertowServletMessages.MESSAGES.twoServletsWithSameMapping(path);
+                    }
                     extensionServlets.put(ext, handler);
                 }
             }
@@ -410,7 +417,7 @@ public class ServletPathMatches {
             if (key.endsWith("/*")) {
                 final String base = key.substring(0, key.length() - 1);
                 if (match == null || base.length() > match.length()) {
-                    if (path.startsWith(base)) {
+                    if (path.startsWith(base) || path.equals(base.substring(0, base.length() - 1))) {
                         match = base.substring(0, base.length() - 1);
                         servlet = entry.getValue();
                     }
